@@ -42,11 +42,21 @@ class ConfigCmd:
 
         self.seed = self.get_random_seed()
 
+        self.processor_count = self.get_processor_count()
+
+        self.known_variables = self.get_known_variables()
+
         self.rankers = self.get_ranker_params()
         self.simulator_name = self.get_simulator()
         if self.simulator_name == 'onlinesimulator':
             self.get_online_simulator_settings()
 
+    def get_processor_count(self):
+        if self._xml_input.findall('processor-count'):
+            return int(self._xml_input.findall('processor-count')[0].text)
+        else:
+            return 1
+
     def get_target(self):
         return self._target
 
@@ -249,6 +259,15 @@ class ConfigCmd:
             else:
                 features_derivation_type.append(self._xml_input.findall('features/derivation-type')[0].text.strip())
         return features_derivation_type
+    def get_known_variables(self):
+        known_variables = []
+        if self._xml_input.findall('features/known-variables'):
+            if len(self._xml_input.findall('features/known-variables')) > 1:
+                for child in self._xml_input.findall('features/known-variables'):
+                    known_variables.append(child.findall('value').text.strip())
+            else:
+                known_variables.append(self._xml_input.findall('features/known-variables')[0].text.strip())
+        return known_variables
 
     def get_metrics(self):
         self.metrics = []
