@@ -4,6 +4,7 @@ from datetime import datetime
 import pandas as pd
 from fair_dynamic_rec.core.util.utils import get_dict_key_from_value
 from tqdm import tqdm
+import sys
 
 class DataCmd:
     """
@@ -39,6 +40,7 @@ class DataCmd:
         self.feature_data = self.load_features(config)
         self.save_item_features(config)
 
+
     # def read_rating_as_pandas_dataframe(self, file_path, separator):
     #     return pd.read_csv(file_path, sep=separator, names=['userid', 'itemid', 'rating'], header=None)
 
@@ -190,7 +192,7 @@ class DataCmd:
                 itemid = instance_separated[0]
                 categories = instance_separated[category_column].split(category_delimiter)
 
-                for category in tqdm(categories):
+                for category in categories:
                     # map categories, and update the dictionary
                     if category not in self.item_topic_mapped_data.keys():
                         self.item_topic_mapped_data[category] = category_index
@@ -443,7 +445,8 @@ class DataCmd:
     def load_features(self, config):
         data_features = {}
         if self.train_data is not None and self.test_data is not None:
-            data_features = self.get_latent_features_by_SVD_LeastSquare(config.get_latent_feature_dim(), config.normalize_latent_feature())
+            # data_features = self.get_latent_features_by_SVD_LeastSquare(config.get_latent_feature_dim(), config.normalize_latent_feature())
+            data_features = self.get_latent_features_by_SVD(config.get_latent_feature_dim(), config.normalize_latent_feature())
         if hasattr(self, 'item_data'):
             if self.item_data is not None:
                 topical_features = self.get_topical_features()
