@@ -6,6 +6,10 @@ from fair_dynamic_rec.core.util.outputs import save_rankings_online_simulator, s
 from tqdm import tqdm
 from datetime import datetime
 from scipy.special import expit
+import sys
+import multiprocessing as mp
+from multiprocessing.pool import ThreadPool
+import itertools
 
 ClickModel = {'cascade': CascadeModel}
 
@@ -17,44 +21,77 @@ class OnlineSimulator():
         # self.data = dataObj.load_features(config)
         # config.get_optimal_ranker()
         # config.get_online_simulator_settings()
+        # print(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ": Start get optimal ranking.")
+        # sys.stdout.flush()
         self.optimal_rankings, self.optimal_scores = self.get_optimal_ranking(config, dataObj)
+        # print(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ": Optimal ranking done.")
+        # sys.stdout.flush()
         # ClickModel = config.feedback_model
         self.click_simulator = ClickModel[config.feedback_model](config)
+        # print(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ": Start get optimal click.")
+        # sys.stdout.flush()
         self.optimal_clicks, self.optimal_rewards = self.click_simulator.get_feedback(self.optimal_scores)
+        # print(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ": Optimal click done.")
+        # sys.stdout.flush()
+        self.user_round = np.zeros(dataObj.n_users)
+        self.overall_rewards = None
 
     def run(self, config, dataObj, rankers):
         print(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ": Simulation started with " + str(config.rounds) + " rounds")
+        sys.stdout.flush()
         overall_optimal_reward = np.zeros(config.rounds)
         self.rankers = rankers#self.set_rankers(config, dataObj)
-        overall_rewards = np.zeros((len(self.rankers), config.rounds))
+        self.overall_rewards = np.zeros((len(self.rankers), config.rounds))
         # regret = np.zeros((len(rankers), config.rounds))
         # overall_rankings = np.zeros((len(rankers), config.rounds))
         # overall_clicks = np.zeros((len(rankers), config.rounds))
-        for i in tqdm(range(config.rounds)):
+        # pool = mp.Pool(config.processor_count)
+        for i in tqdm(range(0,config.rounds,config.processor_count)):
+        # for i in tqdm(range(config.rounds)):
+            pool = ThreadPool(processes=config.processor_count)
             # Select batch of n_users_per_round users
-            user_ids = np.random.choice(range(dataObj.n_users), config.n_users_per_round, replace=False)
-            overall_optimal_reward[i] = np.take(self.optimal_rewards, user_ids).sum()
+            user_ids = [np.random.choice(range(dataObj.n_users), config.n_users_per_round, replace=False) for j in range(config.processor_count)]
+            # Select batch of items, used for factorucb algorithm
+            item_ids = [np.random.choice(range(dataObj.n_items), dataObj.n_users, replace=False) for j in range(config.processor_count)]
+            # user_ids = np.random.choice(range(dataObj.n_users), config.n_users_per_round, replace=False)
+            for j in range(config.processor_count):
+                overall_optimal_reward[i+j] = np.take(self.optimal_rewards, user_ids[j]).sum()
+                self.user_round[user_ids[j]] += 1
+            # overall_optimal_reward[i] = np.take(self.optimal_rewards, user_ids).sum()
+            # self.user_round[user_ids] += 1
             # Iterate over all policies
-            for j in range(len(self.rankers)):
-                # Compute n_recos recommendations
-                rankings = self.rankers[j]["ranker"].get_ranking(user_ids, i)
-                # Compute rewards
-                clicks, rewards = self.compute_rewards(config, dataObj, user_ids, rankings)
-                # Update policy based on rewards
-                self.rankers[j]["ranker"].update(user_ids, rankings, clicks, i)
-                overall_rewards[j, i] = rewards.sum()
-                # regret[j, i] = overall_optimal_reward.sum() - rewards.sum()
-                # overall_rankings[j, i] = rankings
-                # overall_clicks[j, i] = clicks
-                save_rankings_online_simulator(config, dataObj, get_param_config_name(self.rankers[j]["config"]), i, rankings, clicks, user_ids)
-        save_rewards(config, self.rankers, overall_optimal_reward, overall_rewards)
-            # Print info
-            # if i == 0 or (i + 1) % print_every == 0 or i + 1 == n_rounds:
-            #     logger.info("Round: %d/%d. Elapsed time: %f sec." % (i + 1, n_rounds, time.time() - start_time))
-            #     logger.info("Cumulative regrets: \n%s \n" % "\n".join(
-            #         ["	%s : %s" % (policies_name[j], str(np.sum(overall_optimal_reward - overall_rewards[j]))) for j
-            #          in range(n_policies)]))
+            # with mp.Pool(config.processor_count) as pool:
+            args = [(user_ids[j], item_ids[j], i+j, config, dataObj) for j in range(len(user_ids))]
+            results = pool.map(self.run_rankers, args)
+            # for j in range(len(self.rankers)):
+            #     # Compute n_recos recommendations
+            #     rankings = self.rankers[j]["ranker"].get_ranking(user_ids, i)
+            #     # Compute rewards
+            #     clicks, rewards = self.compute_rewards(config, dataObj, user_ids, rankings)
+            #     # Update policy based on rewards
+            #     self.rankers[j]["ranker"].update(user_ids, rankings, clicks, i, self.user_round)
+            #     overall_rewards[j, i] = rewards.sum()
+            #     # regret[j, i] = overall_optimal_reward.sum() - rewards.sum()
+            #     # overall_rankings[j, i] = rankings
+            #     # overall_clicks[j, i] = clicks
+            #     save_rankings_online_simulator(config, dataObj, get_param_config_name(self.rankers[j]["config"]), i, rankings, clicks, user_ids)
+            pool.close()
+        save_rewards(config, self.rankers, overall_optimal_reward, self.overall_rewards)
 
+    def run_rankers(self, args):# user_ids, i, config, dataObj
+        for j in range(len(self.rankers)):
+            # Compute n_recos recommendations
+            rankings = self.rankers[j]["ranker"].get_ranking(args[0], args[1], args[2])
+            # Compute rewards
+            clicks, rewards = self.compute_rewards(args[3], args[4], args[0], rankings)
+            # Update policy based on rewards
+            self.rankers[j]["ranker"].update(args[0], args[1], rankings, clicks, args[2], self.user_round)
+            self.overall_rewards[j, args[2]] = rewards.sum()
+            # regret[j, i] = overall_optimal_reward.sum() - rewards.sum()
+            # overall_rankings[j, i] = rankings
+            # overall_clicks[j, i] = clicks
+            save_rankings_online_simulator(args[3], args[4], get_param_config_name(self.rankers[j]["config"]), args[2], rankings, clicks, args[0])
+        return
 
     def get_optimal_ranking(self, config, dataObj):
         self.user_features, self.item_features = AbstractRanker.get_features_for_optimal_ranker(config, dataObj)
@@ -93,7 +130,7 @@ class OnlineSimulator():
                     coverage = AbstractRanker.ranking_coverage(self.item_features[ranking])
                 optimal_rankings[i] = np.asarray(ranking)
             return optimal_rankings, optimal_scores
-        elif config.optimal_ranker == 'hybrid':
+        elif config.optimal_ranker == 'hybrid' or config.optimal_ranker == 'ears_hybrid':
             n_users = self.user_features.shape[0]
             optimal_rankings, optimal_scores = np.zeros((n_users, config.list_size), dtype=int), np.zeros((n_users, config.list_size))
             for i in range(n_users):
@@ -153,15 +190,31 @@ class OnlineSimulator():
                 scores[i] = batch_user_features[i].dot(topic_coverage.T)
             return self.click_simulator.get_feedback(scores)
         elif config.optimal_ranker == 'hybrid':
+            item_topic = self.item_features[:, :self.item_features.shape[1]-config.get_latent_feature_dim()]
+            item_latent = self.item_features[:, self.item_features.shape[1] - config.get_latent_feature_dim():]
             n_users = batch_user_features.shape[0]
             scores = np.zeros((n_users, config.list_size))
             for i in range(n_users):
-                topic_coverage = self.__convert_to_topic_coverage(dataObj.feature_data['test_item_topical_features'][rankings[i]])
-                item_full_feature = np.zeros((config.list_size, dataObj.feature_data['test_item_topical_features'].shape[1] + dataObj.feature_data['train_item_latent_features'].shape[1]))
-                item_full_feature[:, :dataObj.feature_data['test_item_topical_features'].shape[1]] = topic_coverage
-                item_full_feature[:, dataObj.feature_data['test_item_topical_features'].shape[1]:] = dataObj.feature_data['train_item_latent_features'][rankings[i]]
+                # topic_coverage = self.__convert_to_topic_coverage(dataObj.feature_data['test_item_topical_features'][rankings[i]])
+                topic_coverage = self.__convert_to_topic_coverage(item_topic[rankings[i]])
+                # item_full_feature = np.zeros((config.list_size, dataObj.feature_data['test_item_topical_features'].shape[1] + dataObj.feature_data['train_item_latent_features'].shape[1]))
+                item_full_feature = np.zeros((config.list_size, item_topic.shape[1] + item_latent.shape[1]))
+                # item_full_feature[:, :dataObj.feature_data['test_item_topical_features'].shape[1]] = topic_coverage
+                item_full_feature[:, :item_topic.shape[1]] = topic_coverage
+                # item_full_feature[:, dataObj.feature_data['test_item_topical_features'].shape[1]:] = dataObj.feature_data['train_item_latent_features'][rankings[i]]
+                item_full_feature[:, item_topic.shape[1]:] = item_latent[rankings[i]]
                 scores[i] = batch_user_features[i].dot(item_full_feature.T)
             return self.click_simulator.get_feedback(scores)
+        # elif config.optimal_ranker == 'ears_hybrid':
+        #     n_users = batch_user_features.shape[0]
+        #     scores = np.zeros((n_users, config.list_size))
+        #     for i in range(n_users):
+        #         topic_coverage = self.__convert_to_topic_coverage(dataObj.feature_data['test_item_topical_features'][rankings[i]])
+        #         item_full_feature = np.zeros((config.list_size, dataObj.feature_data['test_item_topical_features'].shape[1] + dataObj.feature_data['train_item_latent_features'].shape[1]))
+        #         item_full_feature[:, :dataObj.feature_data['test_item_topical_features'].shape[1]] = topic_coverage
+        #         item_full_feature[:, dataObj.feature_data['test_item_topical_features'].shape[1]:] = dataObj.feature_data['test_item_latent_features'][rankings[i]]
+        #         scores[i] = batch_user_features[i].dot(item_full_feature.T)
+        #     return self.click_simulator.get_feedback(scores)
 
     def __convert_to_topic_coverage(self, x):
         k, d = x.shape
